#  $Id: Session.pm,v 1.15 2006/02/01 10:03:21 jonas Exp $  -*-perl-*-
package Para::Frame::Session;
#=====================================================================
#
# DESCRIPTION
#   Paranormal.se framework Session class
#
# AUTHOR
#   Jonas Liljegren   <jonas@paranormal.se>
#
# COPYRIGHT
#   Copyright (C) 2004 Jonas Liljegren.  All Rights Reserved.
#
#   This module is free software; you can redistribute it and/or
#   modify it under the same terms as Perl itself.
#
#=====================================================================

use strict;
use CGI;
use FreezeThaw qw( thaw );

BEGIN
{
    our $VERSION  = sprintf("%d.%02d", q$Revision: 1.15 $ =~ /(\d+)\.(\d+)/);
    print "Loading ".__PACKAGE__." $VERSION\n";
}

use Para::Frame::Reload;
use Para::Frame::Time qw( now );
use Para::Frame::User;
use Para::Frame::Route;
use Para::Frame::URI;
use Para::Frame::Utils qw( store_params );

our $SESSION_COOKIE_NAME = 'paraframe-sid';

sub new
{
    my( $class, $req ) = @_;

    my( $s, $sid, $active );

    my $now = now();

    # Each user can have more than one session
    # An explicit logout creates a new session

    # Existing session cookie?
    if( my $s_cookie = $req->cookies->hash->{$SESSION_COOKIE_NAME} )
    {
	$sid = $s_cookie->value;

	# Session still alive?
	if( $s = $Para::Frame::SESSION{$sid} )
	{
	    $s->{'latest'} = $now;
	    return $s;
	}

	$active = 0;
    }
    else # Create new session ID if no cookie active
    {
	# REQNUM is actually the previous number, but it will be
	# unique so that doesn't matter
	#
	$sid = time.'-'.$Para::Frame::REQNUM;
	$req->cookies->add({ $SESSION_COOKIE_NAME => $sid });
	$active = 1;
    }

    $s =  bless
    {
	sid            => $sid,
	active         => $active,
	created        => $now,
	latest         => $now,
	user           => undef,
	debug          => $Para::Frame::CFG->{'debug'},
	template_error => '', # Default
    }, $class;

    # Register s
    $Para::Frame::SESSION{$sid} = $s;

    # Create a route object for the session
    $s->{'route'} = Para::Frame::Route->new();

    $s->init;

    return $s;
}

sub init
{
    # Reimplement this
}

sub new_minimal
{
    my( $class ) = @_;

    # Used for background jobs, not bound to a browser client

    my $sid = time.'-'.$Para::Frame::REQNUM;
    my $now = now();

    my $s =  bless
    {
	sid            => $sid,
	active         => 1,
	created        => $now,
	latest         => $now,
	user           => undef,
	debug          => $Para::Frame::CFG->{'debug'},
	template_error => '', # Default
    }, $class;

    # Register s
    $Para::Frame::SESSION{$sid} = $s;

    return $s;
}

sub after_request
{
    my( $s, $req ) = @_;

    # This is the uri of the previous request. Not necessarily the
    # referer of the next request.

    $s->{'referer'} = Para::Frame::URI->new($req->normalized_uri);
    $s->{'referer'}->query_form(store_params());
}

sub register_result_page
{
    my( $s, $uri, $headers, $page_ref ) = @_;
    # URI should only be the path part
    $s->{'page_result'}{$uri} = [ $headers, $page_ref ];
}

sub id
{
    return $_[0]->{'sid'};
}

sub u
{
    return $_[0]->{'user'};
}

sub user
{
    return $_[0]->{'user'};
}

sub referer
{
    return $_[0]->{'referer'};
}

sub route
{
    return $_[0]->{'route'};
}

sub debug_data
{
    my( $s ) = @_;

    my $out = "";

    $out .= "Session id: $s->{'sid'}\n";
    $out .= "Session created $s->{'created'}\n";
    $out .= "Debug level is $s->{'debug'}\n";
    my $udesig = $s->u->desig;
    $out .= "User is $udesig\n";
    return $out;
}

1;
